/* Parallel Hierarchical Grid -- an adaptive finite element library.
 *
 * Copyright (C) 2005-2010 State Key Laboratory of Scientific and
 * Engineering Computing, Chinese Academy of Sciences. */

/* This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA */

/* $Id: option.h,v 1.26 2008/12/18 01:40:20 zlb Exp $ */

#ifndef PHG_OPTION_H

/* All options are stored in a list 'options'. When parsing a cmdline option,
 * the corresponding variable, *o->var, is updated with the given value.
 * o->var points to the location of the variable with one of the following
 * types:
 *
 * - For VT_INT, o->var is intepreted as (INT *)o->var
 *
 * - For VT_FLOAT, o->var is intepreted as (FLOAT *)o->var
 *
 * - For options without argument (VT_NONE), o->var is intepreted as
 *   (BOOLEAN *)o->var, the cmdline option '-name' sets it to TRUE,
 *   and '+name' sets it to FALSE.
 *
 * - For VT_STRING, o->var is intepreted as (const char **)o->var, i.e.,
 *   it points to the address of the string pointer, calling the option
 *   will modify the pointer to point to the new string which is allocated
 *   and freed by phgOptionXXXXX functions (thus user code should never
 *   free the string after calling phgOptionRegister.
 *
 * - For VT_KEYWORD, o->keys is a NULL terminated list of valid keywords
 *   for this option, and o->var o->var is intepreted as (int *)o->var
 *   which is an index to the list of keywords */

typedef enum {
    VT_NONE, VT_INT, VT_FLOAT, VT_STRING, VT_FILENAME, VT_KEYWORD, VT_HANDLER,
    VT_INCLUDE, VT_TITLE
} VTYPE;

typedef struct {
    char	*name;		/* option name without leading dash */
    char	*help;		/* help text for this option */
    char	**keys;		/* list of key words if type is VT_KEYWORD */
    void 	*var;		/* address of the variable to assign value to
				   It's assumed to have the following type:
					- VT_HANDLER	(OPTION_HANDLER)var
					- VT_FILENAME	(const char **)var
					- VT_STRING	(const char **)var
					- VT_KEYWORD	(int *)var
					- VT_INT	(INT *)var
					- VT_FLOAT	(FLOAT *)var
					- VT_NONE	(BOOLEAN *)var */
    VTYPE	type;		/* type of the variable */
    BOOLEAN	used;		/* whether the option is specified in cmdline */
    BOOLEAN	append;		/* for VT_HANDLER: append or replace */
} OPTION;

/* Option handling function protocol.
 * 'optname' is the name of the option,
 * 'optstr' is the option string (NULL ==> print help).
 * Returns FALSE if error. */
typedef BOOLEAN (*OPTION_HANDLER)(OPTION *o, const char *arg);

#ifdef __cplusplus
extern "C" {
#endif

void phgOptionsRegisterNoArg(const char *name, const char *help, BOOLEAN *var);
void phgOptionsRegisterInt(const char *name, const char *help, INT *var);
void phgOptionsRegisterFloat(const char *name, const char *help, FLOAT *var);
void phgOptionsRegisterString(const char *name, const char *help, char **var);
void phgOptionsRegisterFilename(const char *name, const char *help, char **var);
void phgOptionsRegisterKeyword(const char *name, const char *help,
				const char **keys, int *var);
void phgOptionsRegisterHandler(const char *name, const char *help,
				OPTION_HANDLER func, BOOLEAN append);
void phgOptionsRegisterTitle(const char *str, const char *help,
				const char *category);

void phgOptionsReset(void);
void phgOptionsShowCmdline(void);
void phgOptionsShowUsed(void);
BOOLEAN phgOptionsIfUsed(const char *op_name);
void phgOptionsPrintfHelp(OPTION *o, const char *help);
void phgOptionsHelp(void);
void phgOptionsParseCmdline(int *argc, char ***argv);
void phgOptionsPreset(const char *str);

void phgOptionsSetOptions(const char *str);
BOOLEAN phgOptionsSetNoArg(const char *op_name, BOOLEAN value);
BOOLEAN phgOptionsSetInt(const char *op_name, INT value);
BOOLEAN phgOptionsSetFloat(const char *op_name, FLOAT value);
BOOLEAN phgOptionsSetKeyword(const char *op_name, const char *value);
BOOLEAN phgOptionsSetString(const char *op_name, const char *value);
BOOLEAN phgOptionsSetFilename(const char *op_name, const char *value);
BOOLEAN phgOptionsSetHandler(const char *op_name, const char *value);

void phgOptionsPush(void);
void phgOptionsPop(void);

BOOLEAN phgOptionsGetNoArg(const char *op_name);
INT phgOptionsGetInt(const char *op_name);
FLOAT phgOptionsGetFloat(const char *op_name);
const char *phgOptionsGetKeyword(const char *op_name);
const char *phgOptionsGetString(const char *op_name);
const char *phgOptionsGetFilename(const char *op_name);

#ifdef __cplusplus
}
#endif

#define PHG_OPTION_H
#endif
